% Replicates the Monte Carlo experiment in "The empirical content of games with bounded regressors" by Brendan Kline

clearvars
clearvars -global
close all

maxreps = 1000;
alpha_test = 0.10;
computeStdErrs = 1; % set to 1 to get confidence intervals
N_studied = [500 600 700 800 900 1000];
fileOutput = fopen('output.txt','w');
var_xtild = chol((0.90*eye(4)+0.10*ones(4,4)));
var_eps = chol([0.04 0.01; 0.01 0.04]);

reject_beta_asym = cell(length(N_studied));
reject_alpha_asym = cell(length(N_studied));
reject_delta_asym = cell(length(N_studied));
betahats = cell(length(N_studied));
alphahats = cell(length(N_studied));
deltahats = cell(length(N_studied));
betahat = cell(length(N_studied));
alphahat = cell(length(N_studied));
deltahat = cell(length(N_studied));
for N = N_studied,
	RandStream.setGlobalStream(RandStream('mt19937ar','Seed', 123));

	reject_beta_asym{N}(1:maxreps) = 0;
	reject_alpha_asym{N}(1:maxreps) = 0;
	reject_delta_asym{N}(1:maxreps) = 0;
	
	% parameters of the model, estimator assumes negative interaction effect
		beta1 = [1 0.75];
		beta2 = [1 0.75];
		alpha1 = 0.1;
		alpha2 = 0.1;
		Delta1 = -0.2;
		Delta2 = -0.2;
	% parameters of estimator
		gamma = 6;
		alpha = 4;
		h_int00 = 0.315*N^(-1/16);
		h_int11 = 0.315*N^(-1/16);	
		h_beta0 = (1/N)^(2/(2*alpha+gamma));
		tau = 1.5;
		Ident_mat = speye(N);
		rowsum = sparse(repmat(sparse(eye(N)),1,N)*((1/(N-1))));
		diff_x(1:N^2,1:4) = NaN;
		dK_eval(1:N^2,1:4) = NaN;

	tic
	for reps=1:maxreps,
		% generate the data
			xtild = randn(N,4)*var_xtild;
			x1=1/pi*atan(xtild(:,1:2));
			x2=1/pi*atan(xtild(:,3:4));
			eps=randn(N,2)*var_eps;

			y00=1*(alpha1+x1*beta1'+eps(:,1) <=0).*(alpha2+x2*beta2'+eps(:,2) <=0);
			y11=1*(alpha1+x1*beta1'+Delta1+eps(:,1) >=0).*(alpha2+x2*beta2'+Delta2+eps(:,2) >=0);

		%determine the bandwidth for estimator of beta
		%output of estimator is [dhat_forh betahat2 alphahat deltahat conf_beta conf_alpha conf_delta Q_hat]
			[dhat_tau1, ~, ~, ~, ~, ~, ~, ~] = estimator(x1,x2,y00,y11,tau*h_beta0,h_int00,h_int11,alpha_test,1,0,0,0,rowsum,diff_x,dK_eval);
			[dhat_tau0, ~, ~, ~, ~, ~, ~, Q_hat] = estimator(x1,x2,y00,y11,h_beta0,h_int00,h_int11,alpha_test,1,0,0,1,rowsum,diff_x,dK_eval);

			S_hat = (dhat_tau1-dhat_tau0)/((tau*h_beta0)^alpha-h_beta0^alpha);
			% bandwidth adapted from Powell and Stoker (1996)
			h_beta = (gamma*Q_hat/(alpha*S_hat^2))^(1/(2*alpha+gamma))*(1/N)^(2/(2*alpha+gamma));

		% estimate parameters
			[~, betahat{reps}, alphahat{reps}, deltahat{reps}, conf_beta, conf_alpha, conf_delta, ~] = estimator(x1,x2,y00,y11,h_beta,h_int00,h_int11,alpha_test, 0, computeStdErrs, computeStdErrs, 0,rowsum,diff_x,dK_eval);

			if beta1(2) < conf_beta(1) || beta1(2) > conf_beta(2),
				reject_beta_asym{N}(reps) = 1;
			end;		
			if alpha1 < conf_alpha(1) || alpha1 > conf_alpha(2),
				reject_alpha_asym{N}(reps) = 1;
			end;		
			if Delta1 < conf_delta(1) || Delta1 > conf_delta(2),
				reject_delta_asym{N}(reps) = 1;
			end;

		betahats{N}(reps) = betahat{reps}(1);
		alphahats{N}(reps) = alphahat{reps}(1);
		deltahats{N}(reps) = deltahat{reps}(1);

		curtime = toc;
		if (mod(reps,10) == 0 && reps <= 100) || mod(reps,50) == 0 || reps == maxreps,
			pba = mean(reject_beta_asym{N}(1:reps));
			paa = mean(reject_alpha_asym{N}(1:reps));
			pda = mean(reject_delta_asym{N}(1:reps));
			fprintf(1, 'Sample size: %4.0f and Current rep: %3.0f ...', N, reps);
			fprintf(1, 'Time left: %3.2f minutes ... \n', curtime/reps*(maxreps-reps)/60);
			fprintf(1, 'Current beta rej rate: %3.3f percent (asym) ... \n', pba);
			fprintf(1, 'Current alpha rej rate: %3.3f percent (asym) ... \n', paa);
			fprintf(1, 'Current delta rej rate: %3.3f percent (asym) ... \n', pda);
			fprintf(1, 'Current mean beta (%3.3f): %3.3f... \n', beta1(2), mean(betahats{N}(1:reps)));
			fprintf(1, 'Current mean alpha (%3.3f): %3.3f... \n', alpha1, mean(alphahats{N}(1:reps)));
			fprintf(1, 'Current mean delta (%3.3f): %3.3f... \n', Delta1, mean(deltahats{N}(1:reps)));
			fprintf(1, 'Current variance beta: %3.5f... \n', var(betahats{N}(1:reps)));
			fprintf(1, 'Current variance alpha: %3.5f... \n', var(alphahats{N}(1:reps)));
			fprintf(1, 'Current variance delta: %3.5f... \n', var(deltahats{N}(1:reps)));
			
			if mod(reps,10) == 0,
				fprintf(1, '\n');
			end;
		end;
		if reps == maxreps,
			fprintf(fileOutput, 'Sample size: %4.0f and Current rep: %3.0f ...', N, reps);
			fprintf(fileOutput, 'Time left: %3.2f minutes ... \n', curtime/reps*(maxreps-reps)/60);
			fprintf(fileOutput, 'Current beta rej rate: %3.3f percent (asym) ... \n', pba);
			fprintf(fileOutput, 'Current alpha rej rate: %3.3f percent (asym) ... \n', paa);
			fprintf(fileOutput, 'Current delta rej rate: %3.3f percent (asym) ... \n', pda);
			fprintf(fileOutput, 'Current mean beta (%3.3f): %3.3f... \n', beta1(2), mean(betahats{N}(1:reps)));
			fprintf(fileOutput, 'Current mean alpha (%3.3f): %3.3f... \n', alpha1, mean(alphahats{N}(1:reps)));
			fprintf(fileOutput, 'Current mean delta (%3.3f): %3.3f... \n', Delta1, mean(deltahats{N}(1:reps)));
			fprintf(fileOutput, 'Current variance beta: %3.5f... \n', var(betahats{N}(1:reps)));
			fprintf(fileOutput, 'Current variance alpha: %3.5f... \n', var(alphahats{N}(1:reps)));
			fprintf(fileOutput, 'Current variance delta: %3.5f... \n', var(deltahats{N}(1:reps)));
			fprintf(1, '\n');
			
			fprintf(fileOutput, '$\\beta = %3.3f$ & %3.3f  & %3.3f & %3.3f & %3.3f & %3.3f & %3.3f \\\\ \n', beta1(2), mean(betahats{N}(1:reps)), median(betahats{N}), mean((betahats{N}-beta1(2)).^2), iqr(betahats{N}), var(betahats{N}), pba);
			fprintf(fileOutput, '$\\alpha = %3.3f$ & %3.3f  & %3.3f & %3.3f & %3.3f & %3.3f & %3.3f \\\\ \n', alpha1, mean(alphahats{N}(1:reps)), median(alphahats{N}), mean((alphahats{N}-alpha1).^2), iqr(alphahats{N}), var(alphahats{N}), paa);
			fprintf(fileOutput, '$\\Delta = %3.3f$ & %3.3f  & %3.3f & %3.3f & %3.3f & %3.3f & %3.3f \\\\ \n', Delta1, mean(deltahats{N}(1:reps)), median(deltahats{N}), mean((deltahats{N}-Delta1).^2), iqr(deltahats{N}), var(deltahats{N}), pda);
			fprintf(fileOutput, '\n \n \n \n');
		end;
	end;

	
	fprintf(1, 'Sample size: %4.0f ... \n', N);
	fprintf(1, 'Mean beta (%3.3f): %3.3f \n', beta1(2), mean(betahats{N}));
	fprintf(1, 'Median beta (%3.3f): %3.3f \n', beta1(2), median(betahats{N}));	
	fprintf(1, 'MSE beta: %3.3f \n', mean((betahats{N}-beta1(2)).^2));
	fprintf(1, 'Var beta: %3.3f \n', var(betahats{N}));
	fprintf(1, 'IQR beta: %3.3f \n', iqr(betahats{N}));
	
	
	fprintf(1, 'Mean alpha (%3.3f): %3.3f \n', alpha1, mean(alphahats{N}));
	fprintf(1, 'Median alpha (%3.3f): %3.3f \n', alpha1, median(alphahats{N}));
	fprintf(1, 'MSE alpha: %3.3f \n', mean((alphahats{N}-alpha1).^2));
	fprintf(1, 'Var alpha: %3.3f \n', var(alphahats{N}));
	fprintf(1, 'IRQ alpha: %3.3f \n', iqr(alphahats{N}));
	
	
	fprintf(1, 'Mean delta (%3.3f): %3.3f \n', Delta1, mean(deltahats{N}));
	fprintf(1, 'Median delta (%3.3f): %3.3f \n', Delta1, median(deltahats{N}));
	fprintf(1, 'MSE delta: %3.3f \n', mean((deltahats{N}-Delta1).^2));
	fprintf(1, 'Var delta: %3.3f \n', var(deltahats{N}));
	fprintf(1, 'IQR delta: %3.3f \n', iqr(deltahats{N}));
	
	fprintf(1, '\n \n \n');
end;

% calculate the rate of convergence
for datasize = 1:length(N_studied),
	ybetasamples(datasize,1) = log(sqrt(var(betahats{N_studied(datasize)}(:))));
	yalphasamples(datasize,1) = log(sqrt(var(alphahats{N_studied(datasize)}(:))));
	ydeltasamples(datasize,1) = log(sqrt(var(deltahats{N_studied(datasize)}(:))));
	ybetasamples2(datasize,1) = log(median(abs(betahats{N_studied(datasize)}(:)-beta1(2))));
	yalphasamples2(datasize,1) = log(median(abs(alphahats{N_studied(datasize)}(:)-alpha1)));
	ydeltasamples2(datasize,1) = log(median(abs(deltahats{N_studied(datasize)}(:)-Delta1)));
	ybetasamples3(datasize,1) = log(sqrt(mean((betahats{N_studied(datasize)}(:)-beta1(2)).^2)));
	yalphasamples3(datasize,1) = log(sqrt(mean((alphahats{N_studied(datasize)}(:)-alpha1).^2)));
	ydeltasamples3(datasize,1) = log(sqrt(mean((deltahats{N_studied(datasize)}(:)-Delta1).^2)));
end;
xsamples = [ones(length(N_studied),1) log(N_studied)'];
betastd = inv(xsamples'*xsamples)*xsamples'*ybetasamples
alphastd = inv(xsamples'*xsamples)*xsamples'*yalphasamples
deltastd = inv(xsamples'*xsamples)*xsamples'*ydeltasamples
betamed = inv(xsamples'*xsamples)*xsamples'*ybetasamples2
alphamed = inv(xsamples'*xsamples)*xsamples'*yalphasamples2
deltamed = inv(xsamples'*xsamples)*xsamples'*ydeltasamples2
betamean = inv(xsamples'*xsamples)*xsamples'*ybetasamples3
alphamean = inv(xsamples'*xsamples)*xsamples'*yalphasamples3
deltamean = inv(xsamples'*xsamples)*xsamples'*ydeltasamples3

fprintf(1, '  & Std Error & Root MSE & Med. Abs. \\\\ \n');
fprintf(1, '$\\beta$ & %3.3f & %3.3f & %3.3f \\\\ \n', -betastd(2), -betamean(2), -betamed(2));
fprintf(1, '$\\alpha$ & %3.3f  & %3.3f & %3.3f  \\\\ \n', -alphastd(2), -alphamean(2), -alphamed(2));
fprintf(1, '$\\Delta$ & %3.3f  & %3.3f & %3.3f  \\\\ \n', -deltastd(2), -deltamean(2), -deltamed(2));
fprintf(1, '\n \n \n \n');

fprintf(fileOutput, '  & Std Error & Root MSE & Med. Abs. \\\\ \n');
fprintf(fileOutput, '$\\beta$ & %3.3f & %3.3f & %3.3f \\\\ \n', -betastd(2), -betamean(2), -betamed(2));
fprintf(fileOutput, '$\\alpha$ & %3.3f  & %3.3f & %3.3f  \\\\ \n', -alphastd(2), -alphamean(2), -alphamed(2));
fprintf(fileOutput, '$\\Delta$ & %3.3f  & %3.3f & %3.3f  \\\\ \n', -deltastd(2), -deltamean(2), -deltamed(2));
fprintf(fileOutput, '\n \n \n \n');

fclose(fileOutput);
save('estimates.mat');
